@@ -4,7 +4,7 @@
 ;;
 ;; Orig-Date:    20-Jul-16 at 22:41:34
 ;;
-;; Copyright (C) 2016-2017  Free Software Foundation, Inc.
+;; Copyright (C) 2016-2020  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -198,6 +198,11 @@
   :type 'string
   :group 'hyperbole-button)
 
+(defcustom hibtypes-git-use-magit-flag nil
+  "If magit is available, when activating a git directory button, use Magit rather than Dired."
+  :type 'boolean
+  :group 'hyperbole-button)
+
 (defcustom hibtypes-github-default-project nil
   "Default project name to associate with any Github commit link."
   :type 'string
@@ -281,14 +286,15 @@ listed in `hibtypes-social-inhibit-modes'."
 		      (and (eq major-mode 'markdown-mode)
 			   (hargs:delimited "(" ")"))))
 	     (save-excursion
-	       (if (looking-at "[-#@=/.:_[:alnum:]]")
-		   (skip-chars-backward "-#@=/.:_[:alnum:]"))
+	       (when (looking-at "[-#@=/.:_[:alnum:]]")
+		 (skip-chars-backward "-#@=/.:_[:alnum:]"))
 	       (and (looking-at hibtypes-social-regexp)
-		    ;; Ensure prefix matches to a social web service
+		    ;; Ensure prefix if any matches to a social web service
 		    (save-match-data
 		      (let ((ref (match-string-no-properties 1)))
-			(delq nil (mapcar (lambda (regexp) (string-match regexp ref))
-					  (mapcar #'car hibtypes-social-hashtag-alist)))))
+			(or (string-empty-p ref)
+			    (delq nil (mapcar (lambda (regexp) (string-match regexp ref))
+					      (mapcar #'car hibtypes-social-hashtag-alist))))))
 		    ;; Heuristic to ensure this is not an email address
 		    (save-match-data
 		      (not (and (looking-at mail-address-regexp)
@@ -585,10 +591,10 @@ PROJECT value is provided, it defaults to the value of
 			  (setq reference (substring reference (match-end 0)))))))
 	       (if (and (stringp user) (stringp project))
 		   (funcall hibtypes-social-display-function
-			    (setq a (if reference
+			    (if reference
 				(format url-to-format user project ref-type reference)
-				;; Remove trailing /
-				(substring (format url-to-format user project "" "") 0 -1))))
+			      ;; Remove trailing /
+			      (substring (format url-to-format user project "" "") 0 -1)))
 		 (cond ((and (null user) (null project))
 			(error "(gitlab-reference): Set `hibtypes-gitlab-default-user' and `hibtypes-gitlab-default-project'"))
 		       ((null user)
@@ -802,8 +808,12 @@ PROJECT value is provided, it defaults to the value of
 				  (princ (format "Command: %s\n\n" cmd))
 				  (princ (shell-command-to-string cmd)))))
 			  ;; Project-only reference, run dired on the project home directory
-			  (hpath:display-buffer (dired-noselect
-						 (file-name-as-directory project-dir)))))
+			  (if (and hibtypes-git-use-magit-flag (fboundp #'magit-status-setup-buffer))
+			      (hpath:display-buffer (save-window-excursion
+						      (magit-status-setup-buffer
+						       (file-name-as-directory project-dir))))
+			    (hpath:display-buffer (dired-noselect
+						   (file-name-as-directory project-dir))))))
 		       (t (if project-dir
 			      (error "(git-reference): git project `%s' directory is unreadable or invalid: \"%s\""
 				     project project-dir)
