@@ -104,6 +104,10 @@ Its default value is #'smart-scroll-down."
     ((eq major-mode 'treemacs-mode) .
      ((smart-treemacs) . (smart-treemacs)))
     ;;
+    ;; dired-sidebar-mode
+    ((eq major-mode 'dired-sidebar-mode) .
+     ((smart-dired-sidebar) . (smart-dired-sidebar)))
+    ;;
     ;; Handle Emacs push buttons in buffers
     ((and (fboundp 'button-at) (button-at (point))) .
      ((push-button nil (mouse-event-p last-command-event))
@@ -145,7 +149,7 @@ Its default value is #'smart-scroll-down."
     ;;
     ;; If at the end of a line (eol), invoke the associated Smart Key handler EOL handler.
     ((if (eq major-mode 'kotl-mode)
-	 (and (not (kotl-mode:eobp)) (kotl-mode:eolp))
+	 (and (not (kotl-mode:eobp)) (kotl-mode:eolp t))
        (smart-eolp)) .
        ((funcall action-key-eol-function) . (funcall assist-key-eol-function)))
     ;;
@@ -237,6 +241,12 @@ Its default value is #'smart-scroll-down."
 	 (string-match "^\\*Help\\|Help\\*$" (buffer-name))) .
 	 ((hkey-help-hide) . (hkey-help-hide)))
     ;;
+    ;; Emacs bookmarks menu (bookmark.el)
+    ((eq major-mode 'bookmark-bmenu-mode) .
+     ((bookmark-jump (bookmark-bmenu-bookmark) (hpath:display-buffer-function)) .
+      ;; Below we want the Assist Key to show what the Action Key does.
+      (hkey-help)))
+    ;;
     ;; Pages directory listing mode (page-ext.el)
     ((eq major-mode 'pages-directory-mode) .
      ((pages-directory-goto) . (pages-directory-goto)))
@@ -360,6 +370,9 @@ Its default value is #'smart-scroll-down."
     ((eq major-mode 'gomoku-mode) .
      ((gomoku-human-plays) . (gomoku-human-takes-back)))
     ;;
+    ((string-prefix-p "magit-" (symbol-name major-mode)) .
+     ((smart-magit) . (smart-magit-assist)))
+    ;;
     ;; Support the OO-Browser when available.  It is a separate Emacs
     ;; package not included with Hyperbole.  Hyperbole supplies a stub
     ;; `br-in-browser' test for use here.
@@ -1178,7 +1191,7 @@ buffer and has moved the cursor to the selected buffer."
 ;;; ************************************************************************
 
 (defun smart-imenu-display-item-where (item-name item-pos)
-  "Displays an ITEM-NAME defined at ITEM-POS within the current buffer.
+  "Display an ITEM-NAME defined at ITEM-POS within the current buffer.
 Uses the imenu library and the value of `hpath:display-where' to display."
   (hpath:display-buffer (current-buffer))
   (funcall imenu-default-goto-function item-name item-pos))
@@ -1200,32 +1213,93 @@ sets `hkey-value' to (identifier . identifier-definition-buffer-position)."
        (cdr hkey-value)))
 
 ;; Derived from `imenu' function in the imenu library.
-(defun smart-imenu-item-p (index-item &optional variable-flag)
-  "If INDEX-ITEM is in the current buffer's imenu, return its definition marker position, else nil.
-If INDEX-ITEM is both a function and a variable, the function
+(defun smart-imenu-item-p (index-key &optional variable-flag no-recurse-flag)
+  "If INDEX-KEY is in the current buffer's imenu, return its definition marker position, else nil.
+If INDEX-KEY is both a function and a variable, the function
 definition is used by default; in such a case, when optional
-VARIABLE-FLAG is non-nil, the variable definition is used instead."
-  (when (stringp index-item)
-    ;; Convert a string to an alist element.
-    (if variable-flag
-	;; Match to variable definitions only.
-	(setq index-item (assoc index-item
-				(cdr (assoc "Variables" (imenu--make-index-alist)))))
-      ;; First try to match a function definition and if that fails,
-      ;; then allow variable definitions and other types of matches as well.
-      (let ((alist (imenu--make-index-alist)))
-	(setq index-item (or (assoc index-item alist)
-			     ;; Does nothing unless the dash Emacs Lisp
-			     ;; library is available for the -flatten function.
-			     (and (require 'dash nil t)
-				  (assoc index-item (-flatten alist)))))))
-    (if index-item (cdr index-item))))
+VARIABLE-FLAG is non-nil, the variable definition is used instead.
+NO-RECURSE-FLAG non-nil prevents infinite recursions."
+  (when (stringp index-key)
+    (let (index-item
+	  index-position)
+      ;; Convert a string to an alist element.
+      (if variable-flag
+	  ;; Match to variable definitions only.
+	  (setq index-item (assoc index-key
+				  (cdr (assoc "Variables" (imenu--make-index-alist)))))
+	;; First try to match a function definition and if that fails,
+	;; then allow variable definitions and other types of matches as well.
+	(let ((alist (imenu--make-index-alist)))
+	  (setq index-item (or (assoc index-key alist)
+			       ;; Does nothing unless the dash Emacs Lisp
+			       ;; library is available for the -flatten function.
+			       (and (require 'dash nil t)
+				    (assoc index-key (-flatten alist)))))))
+      (when index-item
+	(setq index-position (when (markerp (cdr index-item))
+			       (marker-position (cdr index-item))))
+	(if (eq index-position 1)
+	    ;; If index position is 1, this means the index markers have
+	    ;; become out of date after buffer edits (likely imenu-auto-rescan
+	    ;; is nil), so do a single rescan to try to fix this.
+	    (unless no-recurse-flag
+	      (setq imenu--index-alist nil)
+	      (imenu--make-index-alist t)
+	      (smart-imenu-item-p index-key variable-flag t))
+	  index-position)))))
 
 ;;; ************************************************************************
 ;;; smart-info functions
 ;;; ************************************************************************
 ;;; In "hmouse-info.el".
 
+
+;;; ************************************************************************
+;;; smart-magit functions
+;;; ************************************************************************
+
+(defun smart-magit ()
+  "Use a key or mouse key to jump to source and to hide/show changes.
+
+Invoked via a key press when in a magit mode and not on a button.
+It assumes that its caller has already checked that the key was
+pressed in an appropriate buffer and has moved the cursor to the
+selected buffer.
+
+If key is pressed:
+ (1) on the last line, quit from the magit mode (\"q\" key binding);
+ (2) at the end of a line, scroll up a windowful;
+ (3) anywhere else, hide/show the thing at point (\"TAB\" key binding)."
+  (interactive)
+  (cond ((last-line-p)
+	 (call-interactively (key-binding "q")))
+	((eolp)
+	 (smart-scroll-up))
+	(t
+	 (call-interactively (key-binding "\t")))))
+
+(defun smart-magit-assist ()
+  "Use an assist key or mouse key to jump to source and to hide/show changes.
+
+Invoked via an assist key press when in a magit mode and not on a
+button.  It assumes that its caller has already checked that the
+assist-key was pressed in an appropriate buffer and has moved the
+cursor to the selected buffer.
+
+If assist-key is pressed:
+ (1) on the last line, quit from the magit mode (\"q\" key binding);
+ (2) at the end of a line, scroll down a windowful;
+ (3) anywhere else, jump to the thing at point (\"RET\" key binding)."
+  (interactive)
+  (cond ((last-line-p)
+	 (call-interactively (key-binding "q")))
+	((eolp)
+	 (smart-scroll-down))
+	(t
+	 (setq current-prefix-arg 1)
+	 (let ((magit-display-file-buffer-function #'hpath:display-buffer))
+	   (call-interactively (key-binding "\r"))))))
+
 ;;; ************************************************************************
 ;;; smart-man functions
 ;;; ************************************************************************
