@@ -16,7 +16,7 @@
 ;;   Simply load this library and you are ready to try it out by
 ;;   double-clicking on various kinds of characters in different buffer major
 ;;   modes.  You'll quickly get the hang of it.  (It also provides a command
-;;   to jump between beginning and end tags within HTML and SGML buffers.)
+;;   to jump between beginning and end tags within HTML, SGML and XML buffers.)
 ;;   
 ;;   A great deal of smarts are built-in so that it does the right thing
 ;;   almost all of the time; many other attempts at similar behavior such as
@@ -63,11 +63,11 @@
 ;;    `hui-select-and-copy-thing' - mark and copy the syntactical unit to the
 ;;      kill ring
 ;;    `hui-select-and-kill-thing' - kill the syntactical unit at point
-;;    `hui-select-goto-matching-tag' - In HTML and SGML modes (actually any
-;;      listed in the variable, `hui-select-markup-modes'), move point to the
+;;    `hui-select-goto-matching-tag' - In modes listed in the variable,
+;;    `hui-select-markup-modes'), move point to the
 ;;      start of the tag paired with the closest tag that point is within or
 ;;      which it precedes, so you can quickly jump back and forth between
-;;      open and close tags.  As noted, in html-mode this is bound to {C-c .}
+;;      open and close tags.  In these modes, this is bound to {C-c .}
 ;;    `hui-select-at-p' - predicate to test if a buffer position (or point) starts
 ;;      or ends a matching syntactical region (excluding a single character).
 ;;
@@ -81,7 +81,7 @@
 ;;      (hui-select-initialize)
 ;;
 ;;   This will autoload this package, bind the selection command to a double
-;;   click of the left mouse button and set up Java, C++ and HTML modes
+;;   click of the left mouse button and set up Java, C++ and `hui-select-markup-modes'
 ;;   for proper entity selection.
 ;;
 ;;   For any version of Emacs you should add the following autoload entries
@@ -107,7 +107,8 @@
 
 (require 'hvar)
 (eval-when-compile
-  (require 'sgml-mode) ;; for html-mode
+  (require 'sgml-mode) ;; for HTML mode
+  (require 'nxml-mode) ;; for XML mode
   (require 'web-mode nil t))
 
 ;;; ************************************************************************
@@ -121,7 +122,7 @@
   :group 'hyperbole-commands)
 
 (defcustom hui-select-markup-modes
-  '(html-mode sgml-mode web-mode)
+  '(html-mode sgml-mode nxml-mode web-mode)
   "*List of markup language modes that use SGML-style <tag> </tag> pairs."
   :type '(repeat (function :tag "Mode"))
   :group 'hyperbole-commands)
@@ -134,7 +135,7 @@
 
 (defcustom hui-select-indent-modes
   (append '(altmath-mode asm-mode csh-mode eiffel-mode ksh-mode
-            math-mode miranda-mode python-mode pascal-mode sather-mode)
+                         math-mode miranda-mode python-mode pascal-mode sather-mode)
 	  hui-select-text-modes)
   "*List of language major modes that use indentation mostly to define syntactic structure."
   :type '(repeat (function :tag "Mode"))
@@ -211,7 +212,7 @@
 
 ;;;###autoload
 (defun hui-select-at-p (&optional pos)
-  "Return non-nil if the character after optional POS (or point) match a syntax entry in `hui-select-syntax-alist'.
+  "Return non-nil if the character after optional POS (or point) matches a syntax entry in `hui-select-syntax-alist'.
 The non-nil value returned is the function to call to select that syntactic unit."
   (interactive "d")
   (unless (and (bobp) (eobp))
@@ -246,13 +247,12 @@ Also, add language-specific syntax setups to aid in thing selection."
   (interactive)
   (unless (boundp 'hyperbole-loading)
     (require 'hyperbole))
-  (if hkey-init
-      (progn
-	(transient-mark-mode 1)
-	(hkey-global-set-key [double-down-mouse-1] nil)
-	(hkey-global-set-key [double-mouse-1] 'hui-select-thing-with-mouse)
-	(hkey-global-set-key [triple-down-mouse-1] nil)
-	(hkey-global-set-key [triple-mouse-1] 'hui-select-thing-with-mouse)))
+  (when hkey-init
+    (transient-mark-mode 1)
+    (hkey-global-set-key [double-down-mouse-1] nil)
+    (hkey-global-set-key [double-mouse-1] 'hui-select-thing-with-mouse)
+    (hkey-global-set-key [triple-down-mouse-1] nil)
+    (hkey-global-set-key [triple-mouse-1] 'hui-select-thing-with-mouse))
   ;;
   ;; These hooks let you select C++ and Java methods and classes by
   ;; double-clicking on the first character of a definition or on its
@@ -275,38 +275,37 @@ Also, add language-specific syntax setups to aid in thing selection."
 			  (modify-syntax-entry ?\} "\)\{" lisp-mode-syntax-table)))
   ;;
   ;; This hook makes tags, comments, sentences and text blocks
-  ;; selectable in HTML mode.  (You would do something similar for
-  ;; SGML mode).
+  ;; selectable in SGML-related modes.
   ;;
   ;; Make tag begin and end delimiters act like grouping characters,
   ;; for easy syntactical selection of tags.
-  (var:add-and-run-hook 'html-mode-hook
-			(lambda ()
-			  (modify-syntax-entry ?<  "(>" html-mode-syntax-table)
-			  (modify-syntax-entry ?>  ")<" html-mode-syntax-table)
-			  (modify-syntax-entry ?\{ "\(\}" html-mode-syntax-table)
-			  (modify-syntax-entry ?\} "\)\{" html-mode-syntax-table)
-			  (modify-syntax-entry ?\" "\"" html-mode-syntax-table)
-			  (modify-syntax-entry ?=  "."  html-mode-syntax-table)
-			  (modify-syntax-entry ?.  "_"  html-mode-syntax-table)
+  (let (hook-sym syntax-table keymap mode-str)
+    (mapc (lambda (mode)
+            (setq mode-str (symbol-name mode)
+                  hook-sym (intern (concat mode-str "-hook"))
+                  syntax-table-sym (intern (concat mode-str "-syntax-table"))
+                  keymap-sym (intern (concat mode-str "-map")))
+            (var:add-and-run-hook hook-sym
+			          `(lambda ()
+                                     (let ((syntax-table (symbol-value ',syntax-table-sym))
+                                           (keymap (symbol-value ',keymap-sym)))
+			               (modify-syntax-entry ?<  "(>" syntax-table)
+			               (modify-syntax-entry ?>  ")<" syntax-table)
+			               (modify-syntax-entry ?\{ "\(\}" syntax-table)
+			               (modify-syntax-entry ?\} "\)\{" syntax-table)
+			               (modify-syntax-entry ?\" "\"" syntax-table)
+			               (modify-syntax-entry ?=  "."  syntax-table)
+			               (modify-syntax-entry ?.  "_"  syntax-table)
+			               (setq sentence-end "\\([^ \t\n\r>]<\\|>\\(<[^>]*>\\)*\\|[.?!][]\"')}]*\\($\\| $\\|\t\\|  \\)\\)[ \t\n]*")
+			               (define-key keymap "\C-c." 'hui-select-goto-matching-tag))))
+            (unless (eq mode 'web-mode)
+              (var:add-and-run-hook
+               hook-sym (lambda ()
 			  (make-local-variable 'comment-start)
 			  (make-local-variable 'comment-end)
 			  (setq comment-start "<!--" comment-end "-->")
-			  (make-local-variable 'sentence-end)
-			  (setq sentence-end "\\([^ \t\n\r>]<\\|>\\(<[^>]*>\\)*\\|[.?!][]\"')}]*\\($\\| $\\|\t\\|  \\)\\)[ \t\n]*")
-			  (define-key html-mode-map "\C-c." 'hui-select-goto-matching-tag)))
-  (var:add-and-run-hook 'web-mode-hook
-			(lambda ()
-			  (modify-syntax-entry ?<  "(>" web-mode-syntax-table)
-			  (modify-syntax-entry ?>  ")<" web-mode-syntax-table)
-			  (modify-syntax-entry ?\{ "\(\}" web-mode-syntax-table)
-			  (modify-syntax-entry ?\} "\)\{" web-mode-syntax-table)
-			  (modify-syntax-entry ?\" "\"" web-mode-syntax-table)
-			  (modify-syntax-entry ?=  "."  web-mode-syntax-table)
-			  (modify-syntax-entry ?.  "_"  web-mode-syntax-table)
-			  (make-local-variable 'sentence-end)
-			  (setq sentence-end "\\([^ \t\n\r>]<\\|>\\(<[^>]*>\\)*\\|[.?!][]\"')}]*\\($\\| $\\|\t\\|  \\)\\)[ \t\n]*")
-			  (define-key web-mode-map "\C-c." 'hui-select-goto-matching-tag))))
+			  (make-local-variable 'sentence-end)))))
+          hui-select-markup-modes)))
 
 (defun hui-select-get-region-boundaries ()
   "Return the (START . END) boundaries of region for `hui-select-thing'."
@@ -343,10 +342,10 @@ interactively, the type of selection is displayed in the minibuffer."
     (when region
       (goto-char (car region))
       (set-mark (cdr region))
-      (if (fboundp 'activate-region) (activate-region))
-      (if (and (boundp 'transient-mark-mode)
-	       transient-mark-mode)
-	  (setq mark-active t))
+      (when (fboundp 'activate-region) (activate-region))
+      (when (and (boundp 'transient-mark-mode)
+	         transient-mark-mode)
+	(setq mark-active t))
       (and (called-interactively-p 'interactive) hui-select-display-type
 	   (message "%s" hui-select-previous))
       (run-hooks 'hui-select-thing-hook)
@@ -448,7 +447,7 @@ does not have a `>' terminator character."
 	      (setq end-point (point))
 	      (when (or (not (re-search-forward "[<>]" nil t))
 			(= (preceding-char) ?<))
-		  ;; No terminator character `>' for end tag
+		;; No terminator character `>' for end tag
 		(setq result end-point)
 		(throw 'done nil))
 	      ;; Skip over nested tags.
@@ -485,14 +484,16 @@ does not have a `>' terminator character."
   "Copy the region surrounding the syntactical unit at point."
   (interactive)
   (let ((bounds (hui-select-boundaries (point))))
-    (if bounds (copy-region-as-kill (car bounds) (cdr bounds)))))
+    (when bounds
+      (copy-region-as-kill (car bounds) (cdr bounds)))))
 
 ;;;###autoload
 (defun hui-select-and-kill-thing ()
   "Kill the region surrounding the syntactical unit at point."
   (interactive "*")
   (let ((bounds (hui-select-boundaries (point))))
-    (if bounds (kill-region (car bounds) (cdr bounds)))))
+    (when bounds
+      (kill-region (car bounds) (cdr bounds)))))
 
 
 ;;
@@ -526,23 +527,23 @@ The character at POS is selected if no other thing is matched."
 	  (mapc
 	   (lambda (sym-func)
 	     (setq region
-		   (if (car (cdr sym-func))
-		       (funcall (car (cdr sym-func)) pos)))
-	     (if (and region (car region)
-		      (hui-select-region-bigger-p
-		       hui-select-old-region region)
-		      (setq region-size
-			    (- (cdr region) (car region)))
-		      (< region-size min-region))
-		 (setq min-region region-size
-		       result
-		       (list
-			;; The actual selection type is
-			;; sometimes different than the one we
-			;; originally tried, so recompute it here.
-			(car (assq hui-select-previous
-				   hui-select-bigger-alist))
-			(car region) (cdr region)))))
+		   (when (car (cdr sym-func))
+		     (funcall (car (cdr sym-func)) pos)))
+	     (when (and region (car region)
+			(hui-select-region-bigger-p
+			 hui-select-old-region region)
+			(setq region-size
+			      (- (cdr region) (car region)))
+			(< region-size min-region))
+	       (setq min-region region-size
+		     result
+		     (list
+		      ;; The actual selection type is
+		      ;; sometimes different than the one we
+		      ;; originally tried, so recompute it here.
+		      (car (assq hui-select-previous
+				 hui-select-bigger-alist))
+		      (car region) (cdr region)))))
 	   hui-select-bigger-alist)
 	  (if result
 	      ;; Returns hui-select-region
@@ -624,15 +625,15 @@ Ignores any match if on an Emacs button and instead returns nil."
   (prog1 (and (hui-select-delimited-thing-call #'hui-select-thing) t)
     ;; If selected region is followed by only whitespace and then a
     ;; newline, add the newline to the region.
-    (if (use-region-p)
-	(if (> (mark) (point))
-	    (save-excursion
-	      (goto-char (mark))
-	      (skip-chars-forward " \t")
-	      (and (char-after) (= ?\n (char-after))
-		   (set-mark (1+ (point)))))
-	  (if (looking-at "[ \t]*\n")
-	      (goto-char (match-end 0)))))))
+    (when (use-region-p)
+      (if (> (mark) (point))
+	  (save-excursion
+	    (goto-char (mark))
+	    (skip-chars-forward " \t")
+	    (and (char-after) (= ?\n (char-after))
+		 (set-mark (1+ (point)))))
+	(when (looking-at "[ \t]*\n")
+	  (goto-char (match-end 0)))))))
 
 (defun hui-select-at-delimited-sexp-p ()
   (unless (eolp)
@@ -649,7 +650,7 @@ Return t if marked, nil otherwise.  If any error occurs such as
   nil."
   (interactive)
   (let ((mark-sexp-func (lambda ()
-			  (if (region-active-p) (deactivate-mark))
+			  (when (region-active-p) (deactivate-mark))
 			  (mark-sexp) t)))
     (condition-case nil
 	(let ((syn-after (char-syntax (char-after)))
@@ -665,7 +666,7 @@ Return t if marked, nil otherwise.  If any error occurs such as
 		      (eq syn-before ?\)))
 		 (backward-sexp)
 		 (funcall mark-sexp-func))))
-	  (error nil))))
+      (error nil))))
 
 ;;; ************************************************************************
 ;;; Private functions
@@ -715,19 +716,19 @@ The non-nil value returned is the function to call to select that syntactic unit
   (unless (and (memq major-mode hui-select-ignore-quoted-sexp-modes)
 	       ;; Ignore quoted identifier sexpressions, like #'function
 	       (char-after) (memq (char-after) '(?# ?\')))
-      (let ((hui-select-char-p)
-	    (hui-select-whitespace)
-	    (hui-select-syntax-alist '((?\" . hui-select-string)
-				       (?\( . hui-select-sexp-start)
-				       (?\$ . hui-select-sexp-start)
-				       (?\' . hui-select-sexp-start)
-				       (?\) . hui-select-sexp-end)
-				       (?\< . hui-select-comment)
-				       ;; Punctuation needed to match
-				       ;; multi-char comment delimiters
-				       (?\. . hui-select-punctuation))))
-	(hui-select-reset)
-	(funcall func))))
+    (let ((hui-select-char-p)
+	  (hui-select-whitespace)
+	  (hui-select-syntax-alist '((?\" . hui-select-string)
+				     (?\( . hui-select-sexp-start)
+				     (?\$ . hui-select-sexp-start)
+				     (?\' . hui-select-sexp-start)
+				     (?\) . hui-select-sexp-end)
+				     (?\< . hui-select-comment)
+				     ;; Punctuation needed to match
+				     ;; multi-char comment delimiters
+				     (?\. . hui-select-punctuation))))
+      (hui-select-reset)
+      (funcall func))))
 
 (defun hui-select-region-bigger-p (old-region new-region)
   "Return t if OLD-REGION is smaller than NEW-REGION and NEW-REGION partially overlaps OLD-REGION, or if OLD-REGION is uninitialized."
@@ -757,13 +758,14 @@ The non-nil value returned is the function to call to select that syntactic unit
 Return the updated cons cell."
   (setcar hui-select-region beginning)
   (setcdr hui-select-region end)
-  (when (and (null beginning) (null end))
+  (unless (or beginning end)
     (setcar hui-select-old-region nil)
     (setcdr hui-select-old-region nil))
-  (if (and (not (eq hui-select-previous 'buffer))
+  (if (and (not (memq hui-select-previous '(buffer markup-pair)))
 	   (integerp beginning) (integerp end)
 	   (= beginning (point-min)) (= end (point-max)))
-      ;; If we selected the whole buffer, make sure that 'thing' type is 'buffer'.
+      ;; If we selected the whole buffer and not matching a markup-pair,
+      ;; make sure that 'thing' type is 'buffer'.
       nil
     hui-select-region))
 
@@ -795,19 +797,20 @@ Returns nil if not within a string."
 	  (if (and (= (point) (1+ bol))
 		   (looking-at (regexp-quote end-delim)))
 	      (setq count (1+ count))
-	    (if (bobp) nil (backward-char 1))))
+	    (unless (bobp)
+              (backward-char 1))))
 	(goto-char opoint)
 	;; If found an even # of starting and ending delimiters before
 	;; opoint, then opoint is at the start of a string, where we want it.
 	(if (zerop (mod count 2))
-	    (if (bobp) nil (backward-char 1))
+	    (unless (bobp)
+              (backward-char 1))
 	  (re-search-backward start-regexp nil t))
 	;; Point is now before the start of the string.
-	(if (re-search-forward start-regexp nil t)
-	    (progn
-	      (setq start (match-beginning 2))
-	      (if (re-search-forward end-regexp nil t)
-		  (hui-select-set-region start (point)))))))))
+	(when (re-search-forward start-regexp nil t)
+	  (setq start (match-beginning 2))
+	  (when (re-search-forward end-regexp nil t)
+	    (hui-select-set-region start (point))))))))
 
 ;;;
 ;;; Code selections
@@ -844,76 +847,75 @@ list, hui-select-brace-modes."
 			      (looking-at defun-prompt-regexp)))
 			(error nil)))))
 	    eod)
-	(if (or at-def-brace
-		;; At the start of a definition:
-		;; Must be at the first non-whitespace character in the line.
-		(and (= (point) (save-excursion (hui-select-back-to-indentation)))
-		     ;; Must be on an alpha or symbol-constituent character.
-		     ;; Also allow ~ for C++ destructors.
-		     (looking-at "[a-zA-z~]\\|\\s_")
-		     ;; Previous line, if any,  must be blank or a comment
-		     ;; start or end or we must be looking at
-		     ;; `defun-prompt-regexp' when at the beginning of the line.
-		     (or (and (stringp defun-prompt-regexp)
-			      (save-excursion
-				(beginning-of-line)
-				(looking-at defun-prompt-regexp)))
-			 (save-excursion
-			   (if (/= (forward-line -1) 0)
-			       t
-			     (hui-select-at-blank-line-or-comment))))))
-	    (progn
-	      (setq hui-select-previous 'brace-def-or-declaration)
-	      ;; Handle declarations and definitions embedded within classes.
-	      (if (and (eq (following-char) ?\{)
-		       (/= (point) (save-excursion
-				     (hui-select-back-to-indentation))))
-		  (setq at-def-brace nil))
-	      ;;
-	      (if at-def-brace nil (beginning-of-line))
-	      (if (and (not at-def-brace)
-		       (stringp defun-prompt-regexp)
-		       (or (looking-at defun-prompt-regexp)
-			   ;; For Java classes mainly
-			   (looking-at "[a-zA-Z_$. \t]+\\s-*\{")))
-		  ;; Mark the declaration or definition
-		  (hui-select-set-region
-		   (point)
-		   (progn (goto-char (match-end 0))
-			  (if (eq (preceding-char) ?\{)
-			      (backward-char 1))
-			  (if (eq (following-char) ?\{)
-			      (forward-list 1)
-			    (search-forward ";" nil t))
-			  (skip-chars-forward " \t")
-			  (skip-chars-forward "\n")
-			  (if (looking-at "^\\s-*$")
-			      (forward-line 1))
-			  (point)))
-		;; Mark function definitions only
-		(setq eod (save-excursion
-			    (condition-case ()
-				(progn
-				  (if (and (eq major-mode 'java-mode)
-					   (fboundp 'id-java-end-of-defun))
-				      (id-java-end-of-defun)
-				    (end-of-defun))
-				  (if (looking-at "^\\s-*$")
-				      (forward-line 1))
-				  (point))
-			      (error (point-max)))))
-		(if (= (following-char) ?\})
-		    ;; Leave point at opening brace.
-		    (goto-char (scan-sexps (1+ (point)) -1)))
-		(if (= (following-char) ?\{)
-		    (progn
-		      (while (and (zerop (forward-line -1))
-				  (not (hui-select-at-blank-line-or-comment))))
-		      (if (hui-select-at-blank-line-or-comment)
-			  (forward-line 1))))
-		;; Mark the whole definition
-		(setq hui-select-previous 'brace-def-or-declaration)
-		(hui-select-set-region (point) eod))))))))
+	(when (or at-def-brace
+		  ;; At the start of a definition:
+		  ;; Must be at the first non-whitespace character in the line.
+		  (and (= (point) (save-excursion (hui-select-back-to-indentation)))
+		       ;; Must be on an alpha or symbol-constituent character.
+		       ;; Also allow ~ for C++ destructors.
+		       (looking-at "[a-zA-z~]\\|\\s_")
+		       ;; Previous line, if any,  must be blank or a comment
+		       ;; start or end or we must be looking at
+		       ;; `defun-prompt-regexp' when at the beginning of the line.
+		       (or (and (stringp defun-prompt-regexp)
+				(save-excursion
+				  (beginning-of-line)
+				  (looking-at defun-prompt-regexp)))
+			   (save-excursion
+			     (if (/= (forward-line -1) 0)
+				 t
+			       (hui-select-at-blank-line-or-comment))))))
+	  (setq hui-select-previous 'brace-def-or-declaration)
+	  ;; Handle declarations and definitions embedded within classes.
+	  (if (and (eq (following-char) ?\{)
+		   (/= (point) (save-excursion
+				 (hui-select-back-to-indentation))))
+	      (setq at-def-brace nil))
+	  ;;
+	  (unless at-def-brace
+	    (beginning-of-line))
+	  (if (and (not at-def-brace)
+		   (stringp defun-prompt-regexp)
+		   (or (looking-at defun-prompt-regexp)
+		       ;; For Java classes mainly
+		       (looking-at "[a-zA-Z_$. \t]+\\s-*\{")))
+	      ;; Mark the declaration or definition
+	      (hui-select-set-region
+	       (point)
+	       (progn (goto-char (match-end 0))
+		      (when (eq (preceding-char) ?\{)
+			(backward-char 1))
+		      (if (eq (following-char) ?\{)
+			  (forward-list 1)
+			(search-forward ";" nil t))
+		      (skip-chars-forward " \t")
+		      (skip-chars-forward "\n")
+		      (when (looking-at "^\\s-*$")
+			(forward-line 1))
+		      (point)))
+	    ;; Mark function definitions only
+	    (setq eod (save-excursion
+			(condition-case ()
+			    (progn
+			      (if (and (eq major-mode 'java-mode)
+				       (fboundp 'id-java-end-of-defun))
+				  (id-java-end-of-defun)
+				(end-of-defun))
+			      (if (looking-at "^\\s-*$")
+				  (forward-line 1))
+			      (point))
+			  (error (point-max)))))
+	    (when (= (following-char) ?\})
+	      ;; Leave point at opening brace.
+	      (goto-char (scan-sexps (1+ (point)) -1)))
+	    (when (= (following-char) ?\{)
+	      (while (and (zerop (forward-line -1))
+			  (not (hui-select-at-blank-line-or-comment))))
+	      (when (hui-select-at-blank-line-or-comment)
+		(forward-line 1)))
+	    ;; Mark the whole definition
+	    (setq hui-select-previous 'brace-def-or-declaration)
+	    (hui-select-set-region (point) eod)))))))
 
 (defun hui-select-indent-def (pos)
   "If POS is at the first alpha character on a line, return (start . end) region.
@@ -924,67 +926,67 @@ list, hui-select-indent-modes."
   (when (memq major-mode hui-select-indent-modes)
     (save-excursion
       (goto-char pos)
-      (if (and
-	   ;; Use this function only if point is on the first non-blank
-	   ;; character of a block, whatever a block is for the current
-	   ;; mode.
-	   (cond ((eq major-mode 'kotl-mode)
-		  (and (looking-at "[1-9*]") (not (kview:valid-position-p))))
-		 ((or (eq major-mode 'outline-mode) selective-display)
-		  (save-excursion (beginning-of-line)
-				  (looking-at outline-regexp)))
-		 ;; After indent in any other mode, must be on an alpha
-		 ;; or symbol-constituent character.
-		 (t (looking-at "[a-zA-z]\\|\\s_")))
-	   ;; Must be at the first non-whitespace character in the line.
-	   (= (point) (save-excursion (hui-select-back-to-indentation))))
-	  (let* ((start-col (current-column))
-		 (opoint (if (eq major-mode 'kotl-mode)
-			     (progn (kotl-mode:to-valid-position) (point))
-			   (beginning-of-line) (point))))
-	    (while
-		(and (zerop (forward-line 1))
-		     (bolp)
-		     (or (progn (hui-select-back-to-indentation)
-				(> (current-column) start-col))
-			 ;; If in a text mode, allow outdenting, otherwise
-			 ;; only include special lines here indented to the
-			 ;; same point as the original line.
-			 (and (or (memq major-mode hui-select-text-modes)
-				  (= (current-column) start-col))
-			      (looking-at
-			       (or (car (cdr
-					 (assq
-					  major-mode
-					  hui-select-indent-non-end-regexp-alist)))
-				   "\\'"))))))
-	    (if (and (looking-at
+      (when (and
+	     ;; Use this function only if point is on the first non-blank
+	     ;; character of a block, whatever a block is for the current
+	     ;; mode.
+	     (cond ((eq major-mode 'kotl-mode)
+		    (and (looking-at "[1-9*]") (not (kview:valid-position-p))))
+		   ((or (eq major-mode 'outline-mode) selective-display)
+		    (save-excursion (beginning-of-line)
+				    (looking-at outline-regexp)))
+		   ;; After indent in any other mode, must be on an alpha
+		   ;; or symbol-constituent character.
+		   (t (looking-at "[a-zA-z]\\|\\s_")))
+	     ;; Must be at the first non-whitespace character in the line.
+	     (= (point) (save-excursion (hui-select-back-to-indentation))))
+	(let* ((start-col (current-column))
+	       (opoint (if (eq major-mode 'kotl-mode)
+			   (progn (kotl-mode:to-valid-position) (point))
+			 (beginning-of-line) (point))))
+	  (while
+	      (and (zerop (forward-line 1))
+		   (bolp)
+		   (or (progn (hui-select-back-to-indentation)
+			      (> (current-column) start-col))
+		       ;; If in a text mode, allow outdenting, otherwise
+		       ;; only include special lines here indented to the
+		       ;; same point as the original line.
+		       (and (or (memq major-mode hui-select-text-modes)
+				(= (current-column) start-col))
+			    (looking-at
+			     (or (car (cdr
+				       (assq
+					major-mode
+					hui-select-indent-non-end-regexp-alist)))
+				 "\\'"))))))
+	  (when (and (looking-at
 		      (or (car (cdr (assq major-mode
 					  hui-select-indent-end-regexp-alist)))
 			  "\\'"))
 		     (or (memq major-mode hui-select-text-modes)
 			 (= (current-column) start-col)))
-		(forward-line 1))
-	    (beginning-of-line)
-	    ;; Mark the whole definition
-	    (setq hui-select-previous 'indent-def)
-	    (hui-select-set-region opoint (point)))))))
+	    (forward-line 1))
+	  (beginning-of-line)
+	  ;; Mark the whole definition
+	  (setq hui-select-previous 'indent-def)
+	  (hui-select-set-region opoint (point)))))))
 
 (defun hui-select-symbol (pos)
   "Return (start . end) of a symbol at POS."
   (or (hui-select-markup-pair pos)
-   ;; Test for indented def here since might be on an '*' representing
-   ;; an outline entry, in which case we mark entries as indented blocks.
-   (hui-select-indent-def pos)
-   (save-excursion
-     (if (memq (char-syntax (if (eobp) (preceding-char) (char-after pos)))
-	       '(?w ?_))
-	 (progn (setq hui-select-previous 'symbol)
-		(condition-case ()
-		    (let ((end (scan-sexps pos 1)))
-		      (hui-select-set-region
-		       (min pos (scan-sexps end -1)) end))
-		  (error nil)))))))
+      ;; Test for indented def here since might be on an '*' representing
+      ;; an outline entry, in which case we mark entries as indented blocks.
+      (hui-select-indent-def pos)
+      (save-excursion
+        (when (memq (char-syntax (if (eobp) (preceding-char) (char-after pos)))
+	          '(?w ?_))
+	  (setq hui-select-previous 'symbol)
+	  (condition-case ()
+	      (let ((end (scan-sexps pos 1)))
+		(hui-select-set-region
+		 (min pos (scan-sexps end -1)) end))
+	    (error nil))))))
 
 (defun hui-select-sexp-start (pos)
   "Return (start . end) of sexp starting at POS."
@@ -1025,7 +1027,7 @@ list, hui-select-indent-modes."
     (goto-char pos)
     (condition-case ()
 	(hui-select-set-region (progn (backward-up-list 1) (point))
-			      (progn (forward-list 1) (point)))
+			       (progn (forward-list 1) (point)))
       (error nil))))
 
 (defun hui-select-preprocessor-def (pos)
@@ -1039,17 +1041,16 @@ included in the list, hui-select-brace-modes."
     (setq hui-select-previous 'preprocessor-def)
     (save-excursion
       (goto-char pos)
-      (if (and (= (following-char) ?#)
-	       ;; Must be at the first non-whitespace character in the line.
-	       (= (point) (save-excursion (hui-select-back-to-indentation))))
-	  (progn
-	    ;; Skip past continuation lines that end with a backslash.
-	    (while (and (looking-at ".*\\\\\\s-*$")
-			(zerop (forward-line 1))))
-	    (forward-line 1)
-	    ;; Include one trailing blank line, if any.
-	    (if (looking-at "^[ \t\n\r]*$") (forward-line 1))
-	    (hui-select-set-region pos (point)))))))
+      (when (and (= (following-char) ?#)
+		 ;; Must be at the first non-whitespace character in the line.
+		 (= (point) (save-excursion (hui-select-back-to-indentation))))
+	;; Skip past continuation lines that end with a backslash.
+	(while (and (looking-at ".*\\\\\\s-*$")
+		    (zerop (forward-line 1))))
+	(forward-line 1)
+	;; Include one trailing blank line, if any.
+	(if (looking-at "^[ \t\n\r]*$") (forward-line 1))
+	(hui-select-set-region pos (point))))))
 
 ;; Allow punctuation marks not followed by white-space to include
 ;; the previous and subsequent sexpression.  Useful in contexts such as
@@ -1078,10 +1079,10 @@ included in the list, hui-select-brace-modes."
   (setq hui-select-previous 'comment)
   (save-excursion
     (goto-char pos)
-    (let ((start-regexp  (if (stringp comment-start)
-			     (regexp-quote comment-start)))
-	  (end-regexp    (if (stringp comment-end)
-			     (regexp-quote comment-end)))
+    (let ((start-regexp  (when (stringp comment-start)
+			   (regexp-quote comment-start)))
+	  (end-regexp    (when (stringp comment-end)
+			   (regexp-quote comment-end)))
 	  bolp)
       (cond
        ;; Beginning of a comment
@@ -1106,15 +1107,15 @@ included in the list, hui-select-brace-modes."
 			(setq count (1+ count))
 		      ;; End comment
 		      (setq count (1- count))
-		      (if (= count 0)
-			  (progn
-			    (if (looking-at "[ \t]*[\n\r]")
-				;; Don't include final newline unless the
-				;; comment is first thing on its line.
-				(goto-char (if bolp (match-end 0)
-					     (1- (match-end 0)))))
-			    (throw 'done (hui-select-set-region
-					  pos (point))))))))))))
+		      (when (= count 0)
+			(when (looking-at "[ \t]*[\n\r]")
+			  ;; Don't include final newline unless the
+			  ;; comment is first thing on its line.
+			  (goto-char (if bolp
+					 (match-end 0)
+				       (1- (match-end 0)))))
+			(throw 'done (hui-select-set-region
+				      pos (point)))))))))))
        ;; End of a comment
        ((and (stringp comment-end)
 	     (not (string-equal comment-end ""))
@@ -1122,8 +1123,8 @@ included in the list, hui-select-brace-modes."
 		 (and (skip-chars-backward comment-end)
 		      (looking-at end-regexp))))
 	(goto-char (match-end 0))
-	(if (looking-at "[ \t]*[\n\r]")
-	    (goto-char (match-end 0)))
+	(when (looking-at "[ \t]*[\n\r]")
+	  (goto-char (match-end 0)))
 	(setq pos (point))
 	(skip-chars-forward " \t")
 	;; Skip over nested comments.
@@ -1135,14 +1136,14 @@ included in the list, hui-select-brace-modes."
 		  (setq count (1+ count))
 		;; Begin comment
 		(setq count (1- count))
-		(if (= count 0)
-		    (progn
-		      (skip-chars-backward " \t")
-		      ;; Don't include final newline unless the comment is
-		      ;; first thing on its line.
-		      (if (bolp) nil (setq pos (1- pos)))
-		      (throw 'done (hui-select-set-region
-				    (point) pos)))))))))))))
+		(when (= count 0)
+		  (skip-chars-backward " \t")
+		  ;; Don't include final newline unless the comment is
+		  ;; first thing on its line.
+		  (unless (bolp)
+		    (setq pos (1- pos)))
+		  (throw 'done (hui-select-set-region
+				(point) pos))))))))))))
 
 ;;;
 ;;; Textual selections
@@ -1188,7 +1189,7 @@ Delimiters may be single, double or open and close quotes."
     (goto-char pos)
     (condition-case ()
 	(hui-select-set-region (progn (backward-sentence) (point))
-			      (progn (forward-sentence) (point)))
+			       (progn (forward-sentence) (point)))
       (error nil))))
 
 (defun hui-select-whitespace (pos)
@@ -1202,13 +1203,13 @@ the line.  Then return all of it."
 	(hui-select-page pos)
       (let ((end (progn (skip-chars-forward " \t") (point)))
 	    (start (progn (skip-chars-backward " \t") (point))))
-	(if (looking-at "[ \t]")
-	    (if (or (bolp) (= (1+ start) end))
-		(hui-select-set-region start end)
-	      (hui-select-set-region (1+ start) end)))))))
+	(when (looking-at "[ \t]")
+	  (if (or (bolp) (= (1+ start) end))
+	      (hui-select-set-region start end)
+	    (hui-select-set-region (1+ start) end)))))))
 
 (defun hui-select-markup-pair (pos)
-  "Return (start . end) of region between the opening and closing of an HTML or SGML tag pair, one of which is at POS.
+  "Return (start . end) of region between the opening and closing of an HTML, XML or SGML tag pair, one of which is at POS.
 The major mode for each language that uses such tags must be included in the
 list, hui-select-markup-modes."
   (when (memq major-mode hui-select-markup-modes)
@@ -1222,87 +1223,88 @@ list, hui-select-markup-modes."
 	  bolp
 	  opoint)
       (save-excursion
-	(catch 'done
-	  (goto-char pos)
-	  (cond
-	   ;; Beginning of a tag pair
-	   ((looking-at "<[^/][^<> \t\n\r]*")
-	    (setq start-regexp (regexp-quote (match-string 0))
-		  end-regexp   (concat "</" (substring start-regexp 1)))
-	    (setq pos (point))
-	    (skip-chars-backward " \t")
-	    (setq bolp (bolp)
-		  pos-with-space (point))
-	    ;; Skip over nested tags.
-	    (let ((count 0)
-		  (regexp (concat start-regexp "\\|" end-regexp)))
-	      (while (and (>= count 0)
-			  (re-search-forward regexp nil t))
-		(if (/= (char-after (1+ (match-beginning 0))) ?/)
-		    ;; Start tag
-		    (setq count (1+ count))
-		  ;; Move past end tag terminator
-		  (setq opoint (point))
-		  (if (or (not (re-search-forward "[<>]" nil t))
-			  (= (preceding-char) ?<))
-		      (progn (setq result opoint)
-			     (throw 'done nil)))
-		  (setq count (1- count))
-		  (if (= count 0)
-		      (progn
-			(if (looking-at "[ \t]*[\n\r]")
-			    ;; Don't include final newline unless the
-			    ;; start tag was the first thing on its line.
-			    (if bolp
-				(progn (goto-char (match-end 0))
-				       ;; Include leading space since the
-				       ;; start and end tags begin and end
-				       ;; lines.
-				       (setq pos pos-with-space))
-			      (goto-char (1- (match-end 0)))))
-			(setq result (hui-select-set-region pos (point)))
-			(throw 'done nil)))))))
-	   ;;
-	   ;; End of a tag pair
-	   ((or (looking-at "</[^\> \t\n\r]+")
-		(and (skip-chars-backward "\<")
-		     (looking-at "</[^\> \t\n\r]+")))
-	    (goto-char (match-end 0))
-	    (setq end-regexp (regexp-quote (match-string 0))
-		  start-regexp   (concat "\<" (substring end-regexp 2)))
-	    (setq opoint (point))
-	    (if (or (not (re-search-forward "[<>]" nil t))
-		    (= (preceding-char) ?<))
-		(progn (setq result opoint)
-		       (throw 'done nil)))
-	    (setq pos (point))
-	    (if (looking-at "[ \t]*[\n\r]")
-		(setq pos-with-space (match-end 0)))
-	    ;; Skip over nested tags.
-	    (let ((count 0)
-		  (regexp (concat start-regexp "\\|" end-regexp)))
-	      (while (and (>= count 0)
-			  (re-search-backward regexp nil t))
-		(if (= (char-after (1+ (point))) ?/)
-		    ;; End tag
-		    (setq count (1+ count))
-		  ;; Start tag
-		  (setq count (1- count))
-		  (if (= count 0)
-		      (progn
-			(if pos-with-space
+        (setq result
+	      (catch 'done
+	        (goto-char pos)
+	        (cond
+	         ;; Beginning of a tag pair
+	         ((looking-at "<[^/][^<> \t\n\r]*")
+	          (setq start-regexp (regexp-quote (match-string 0))
+		        end-regexp   (concat "</" (substring start-regexp 1)))
+	          (setq pos (point))
+	          (skip-chars-backward " \t")
+	          (setq bolp (bolp)
+		        pos-with-space (point))
+	          ;; Skip over nested tags.
+	          (let ((count 0)
+		        (regexp (concat start-regexp "\\|" end-regexp)))
+	            (while (and (>= count 0)
+			        (re-search-forward regexp nil t))
+		      (if (/= (char-after (1+ (match-beginning 0))) ?/)
+		          ;; Start tag
+		          (setq count (1+ count))
+		        ;; Move past end tag terminator
+		        (setq opoint (point))
+		        (when (or (not (re-search-forward "[<>]" nil t))
+			          (= (preceding-char) ?<))
+			  (throw 'done opoint))
+		        (setq count (1- count))
+		        (when (= count 0)
+                          (cond ((eobp)
+                                 (if bolp
+				     ;; Include leading space since the
+				     ;; start and end tags begin and end
+				     ;; lines.
+				     (setq pos pos-with-space)
+                                   (goto-char (1- (point)))))
+			        ((looking-at "[ \t]*[\n\r]")
+			         ;; Don't include final newline unless the
+			         ;; start tag was the first thing on its line.
+			         (if bolp
+				     (progn (goto-char (match-end 0))
+				            ;; Include leading space since the
+				            ;; start and end tags begin and end
+				            ;; lines.
+				            (setq pos pos-with-space))
+			           (goto-char (1- (match-end 0))))))
+			  (throw 'done (hui-select-set-region pos (point))))))))
+	         ;;
+	         ;; End of a tag pair
+	         ((or (looking-at "</[^\> \t\n\r]+")
+		      (and (skip-chars-backward "\<")
+		           (looking-at "</[^\> \t\n\r]+")))
+	          (goto-char (match-end 0))
+	          (setq end-regexp (regexp-quote (match-string 0))
+		        start-regexp   (concat "\<" (substring end-regexp 2))
+	                opoint (point))
+	          (when (or (not (re-search-forward "[<>]" nil t))
+		            (= (preceding-char) ?<))
+		    (throw 'done opoint))
+	          (setq pos (point))
+	          (when (looking-at "[ \t]*[\n\r]")
+		    (setq pos-with-space (match-end 0)))
+	          ;; Skip over nested tags.
+	          (let ((count 0)
+		        (regexp (concat start-regexp "\\|" end-regexp)))
+	            (while (and (>= count 0)
+			        (re-search-backward regexp nil t))
+		      (if (= (char-after (1+ (point))) ?/)
+		          ;; End tag
+		          (setq count (1+ count))
+		        ;; Start tag
+		        (setq count (1- count))
+		        (when (= count 0)
+			  (when pos-with-space
 			    ;; Newline found after original end tag.
-			    (progn
-			      (skip-chars-backward " \t")
-			      (if (bolp)
-				  ;; Don't include final newline unless the
-				  ;; start tag is the first thing on its line.
-				  (setq pos pos-with-space)
-				(setq pos (1- pos-with-space))
-				;; Don't include non-leading space.
-				(skip-chars-forward " \t"))))
-			(setq result (hui-select-set-region (point) pos))
-			(throw 'done nil))))))))))
+			    (skip-chars-backward " \t")
+			    (if (bolp)
+				;; Don't include final newline unless the
+				;; start tag is the first thing on its line.
+				(setq pos pos-with-space)
+			      (setq pos (1- pos-with-space))
+			      ;; Don't include non-leading space.
+			      (skip-chars-forward " \t")))
+			  (throw 'done (hui-select-set-region (point) pos)))))))))))
       (if (integerp result)
 	  (progn (goto-char result)
 		 (error "(hui-select-markup-pair): Add a terminator character for this end tag"))
@@ -1326,9 +1328,10 @@ list, hui-select-markup-modes."
   (setq hui-select-previous 'paragraph)
   (save-excursion
     (goto-char pos)
-    (if (looking-at paragraph-start) (forward-paragraph))
+    (when (looking-at paragraph-start)
+      (forward-paragraph))
     (hui-select-set-region (progn (backward-paragraph) (point))
-			  (progn (forward-paragraph) (point)))))
+			   (progn (forward-paragraph) (point)))))
 
 (defun hui-select-page (pos)
   "Return (start . end) of the page preceding POS."
@@ -1336,7 +1339,7 @@ list, hui-select-markup-modes."
   (save-excursion
     (goto-char pos)
     (hui-select-set-region (progn (backward-page) (point))
-			  (progn (forward-page) (point)))))
+			   (progn (forward-page) (point)))))
 
 (defun hui-select-buffer (pos)
   "Return (start . end) of the buffer at POS."
