@@ -23,6 +23,10 @@
 ;; ======================================================================================================
 ;; Drag from thing start or end    Yank thing at release       Kill thing and yank at release
 ;;
+;; Drag from bottom Modeline       Reposition frame as         <- same
+;; in frame with non-nil           drag happens
+;; drag-with-mode-line param
+
 ;; Drag from shared window side
 ;;   or from left of scroll bar    Resize window width         <- same
 ;; Modeline vertical drag          Resize window height        <- same
@@ -72,7 +76,7 @@
 
 (eval-when-compile (defvar assist-flag nil)) ;; Silences free variable compiler warnings
 (require 'hycontrol)
-;; For momentary highlighting of buffer/file item lines.
+;; If installed, use pulse library for momentary highlighting of buffer/file item lines.
 (require 'pulse nil t)
 
 ;;; ************************************************************************
@@ -104,7 +108,7 @@ of screen control commands."
   :group 'hyperbole-keys)
 
 (defun hmouse-map-modes-to-form (mode-forms)
-  "Maps over a sequence of (major-mode(s) form-to-eval) lists; returns items with a single major-mode in the car, (major-mode form-to-eval)."
+  "Map over a sequence of (major-mode(s) form-to-eval) lists; return items with a single `major-mode' in the car, (major-mode form-to-eval)."
   (apply 'nconc
 	 (mapcar (lambda (modes-form)
 		   (if (sequencep (car modes-form))
@@ -133,7 +137,7 @@ of screen control commands."
 		    (error "(hmouse-item-to-window): %s the treemacs package for item dragging support"
 			   (if (fboundp 'treemacs) "Update" "Install")))))
   "List of (major-mode lisp-form) lists.
-The car of an item must be a major-mode symbol.  The cadr of an item
+The car of an item must be a `major-mode' symbol.  The cadr of an item
 is a Lisp form to evaluate to get the item name at point (typically a
 buffer, file or directory name whose contents will be displayed in the
 drag release window.")
@@ -145,10 +149,10 @@ drag release window.")
   :group 'hyperbole-keys)
 
  ;; Mats Lidell says this should be 10 characters for GNU Emacs.
-(defvar hmouse-edge-sensitivity (if (featurep 'xemacs) 3 10)
+(defvar hmouse-edge-sensitivity 10
   "*Number of characters from window edges within which a click is considered at an edge.")
 
-(defvar hmouse-side-sensitivity (if (featurep 'xemacs) 1 5)
+(defvar hmouse-side-sensitivity 5
   "*Characters in either direction from window side within which a click is considered on the side.")
 
 (defvar hmouse-x-drag-sensitivity 5
@@ -181,7 +185,7 @@ drag release window.")
 	     ;; If click in the minibuffer when it is not active (blank),
 	     ;; display the Hyperbole minibuffer menu or popup the jump menu.
 	     ((hmouse-inactive-minibuffer-p) .
-	      ((funcall action-key-minibuffer-function) . 
+	      ((funcall action-key-minibuffer-function) .
 	       (funcall assist-key-minibuffer-function)))
 	     ((hmouse-drag-thing) .
 	      ((hmouse-yank-region) . (hmouse-kill-and-yank-region)))
@@ -197,8 +201,9 @@ drag release window.")
 	     ;;   ((and (hmouse-modeline-depress) (hmouse-drag-between-frames)) .
 	     ;;    ((hmouse-clone-window-to-frame) . (hmouse-move-window-to-frame)))
 	     ;;
-	     ;; Drag with release on a Modeline
-	     ((and (hmouse-modeline-release) (not (hmouse-modeline-click))) .
+	     ;; Drag from within a window (not a Modeline) with release on a Modeline
+	     ((and (not (hmouse-modeline-depress)) (hmouse-modeline-release)
+		   (not (hmouse-modeline-click))) .
 	      ((or (hmouse-drag-item-to-display t) (hmouse-buffer-to-window t)) .
 	       (hmouse-swap-buffers)))
 	     ;; Non-vertical Modeline drag between windows
@@ -247,10 +252,12 @@ drag release window.")
 ;;; Public functions
 ;;; ************************************************************************
 
-(defun hmouse-at-item-p ()
+(defun hmouse-at-item-p (start-window)
   "Return t if point is on an item draggable by Hyperbole, otherwise nil."
-  (let* ((buf (and (window-live-p action-key-depress-window) (window-buffer action-key-depress-window)))
-	 (mode (and buf (buffer-local-value 'major-mode buf))))
+  (let* ((buf (when (window-live-p start-window)
+		(window-buffer start-window)))
+	 (mode (when buf
+		 (buffer-local-value 'major-mode buf))))
     (and buf (with-current-buffer buf
 	       ;; Point must be on an item, not after one
 	       (not (looking-at "\\s-*$")))
@@ -356,7 +363,7 @@ Delimited constructs include lists, comments, strings,
       (if (and (not (hmouse-use-region-p)) (hui-select-at-delimited-thing-p)
 	       (or (markerp depress-args) (markerp release-args)
 		   (and (not (or (hmouse-drag-window-side) (hmouse-modeline-depress)))
-			(or (hmouse-drag-between-windows) (hmouse-drag-vertically) 
+			(or (hmouse-drag-between-windows) (hmouse-drag-vertically)
 			    (hmouse-drag-horizontally) (hmouse-drag-diagonally))))
 	       (let ((start-buf (window-buffer (smart-window-of-coords depress-args)))
 		     (end-buf (window-buffer (smart-window-of-coords release-args)))
@@ -398,7 +405,7 @@ Signals an error if the depress buffer is read-only."
       ;; In this case, we want an error that will terminate execution so that
       ;; hkey-region is not reset to nil.  This allows the user to fix the
       ;; problem and then to try killing again.
-      (error "(hmouse-kill-region): Use {%s} to enable killing from this buffer."
+      (error "(hmouse-kill-region): Use {%s} to enable killing from this buffer"
 	     (hmouse-read-only-toggle-key))
     (kill-region (or hkey-value (point)) (mark))))
 
@@ -412,9 +419,9 @@ Signals an error if either depress or release buffer is read-only."
 	;; In this case, we want an error that will terminate execution so that
 	;; hkey-region is not reset to nil.  This allows the user to fix the
 	;; problem and then to try killing again.
-	(error "(hmouse-kill-and-yank-region): Use {%s} to enable killing from this buffer."
+	(error "(hmouse-kill-and-yank-region): Use {%s} to enable killing from this buffer"
 	       (hmouse-read-only-toggle-key))
-      ;; Depress and release may be in the same buffer, in which case, 
+      ;; Depress and release may be in the same buffer, in which case,
       ;; save the release point that may change as a result of
       ;; the kill; also, before the kill, restore the point to where it
       ;; was when the region was set.
@@ -425,7 +432,7 @@ Signals an error if either depress or release buffer is read-only."
 	    ;; In this case, we want an error that will terminate execution so that
 	    ;; hkey-region is not reset to nil.  This allows the user to fix the
 	    ;; problem and then to try yanking again.
-	    (error "(hmouse-kill-and-yank-region): Use {%s} to enable yanking into this buffer."
+	    (error "(hmouse-kill-and-yank-region): Use {%s} to enable yanking into this buffer"
 		   (hmouse-read-only-toggle-key))
 	  ;; Region may be in another buffer, so move there if so.
 	  (hmouse-goto-region-prev-point)
@@ -453,7 +460,7 @@ Signals an error if the buffer is read-only."
 	;; In this case, we want an error that will terminate execution so that
 	;; hkey-region is not reset to nil.  This allows the user to fix the
 	;; problem and then to try yanking again.
-	(error "(hmouse-yank-region): Use {%s} to enable yanking into this buffer."
+	(error "(hmouse-yank-region): Use {%s} to enable yanking into this buffer"
 	       (hmouse-read-only-toggle-key))
       ;; Permanently return to release point
       (let ((release-window (if assist-flag assist-key-release-window action-key-release-window)))
@@ -465,7 +472,7 @@ Signals an error if the buffer is read-only."
 	(error nil)))))
 
 (defun hmouse-drag-between-frames ()
-  "Returns non-nil if last Action Key depress and release were in different frames.
+  "Return non-nil if last Action Key depress and release were in different frames.
 If free variable `assist-flag' is non-nil, uses Assist Key."
   (if assist-flag
       (and (window-live-p assist-key-depress-window)
@@ -478,7 +485,7 @@ If free variable `assist-flag' is non-nil, uses Assist Key."
 		  (window-frame action-key-release-window))))))
 
 (defun hmouse-drag-between-windows ()
-  "Returns non-nil if last Action Key depress and release were in different windows.
+  "Return non-nil if last Action Key depress and release were in different windows.
 If free variable `assist-flag' is non-nil, uses Assist Key."
   (if assist-flag
       (and (window-live-p assist-key-depress-window)
@@ -490,7 +497,7 @@ If free variable `assist-flag' is non-nil, uses Assist Key."
 	 (not (eq action-key-depress-window action-key-release-window)))))
 
 (defun hmouse-drag-same-window ()
-  "Returns non-nil if last Action Key depress and release were in the same window.
+  "Return non-nil if last Action Key depress and release were in the same window.
 If free variable `assist-flag' is non-nil, uses Assist Key."
   (if assist-flag
       (and (window-live-p assist-key-depress-window)
@@ -501,7 +508,7 @@ If free variable `assist-flag' is non-nil, uses Assist Key."
 	 (eq action-key-depress-window action-key-release-window))))
 
 (defun hmouse-drag-outside-all-windows ()
-  "Returns non-nil if last Action Key release was outside of an Emacs window.
+  "Return non-nil if last Action Key release was outside of an Emacs window.
 If free variable `assist-flag' is non-nil, uses Assist Key."
   (if assist-flag
       (and (window-live-p assist-key-depress-window) (not assist-key-release-window))
@@ -522,12 +529,12 @@ not on an item, then nil.
 
 See `hmouse-drag-item-mode-forms' for how to allow for draggable
 items in other modes."
-  (when (hmouse-at-item-p)
+  (when (hmouse-at-item-p action-key-depress-window)
     (hmouse-item-to-window new-window)
     t))
 
 (defun hmouse-drag-diagonally ()
-  "Returns non-nil iff last Action Key use was a diagonal drag within a single window.
+  "Return non-nil iff last Action Key use was a diagonal drag within a single window.
 If free variable `assist-flag' is non-nil, uses Assist Key.
 Value returned is nil if not a diagonal drag, or one of the following symbols
 depending on the direction of the drag: southeast, southwest, northwest, northeast."
@@ -558,7 +565,7 @@ depending on the direction of the drag: southeast, southwest, northwest, northea
 		   'southwest 'northwest)))))))
 
 (defun hmouse-drag-horizontally ()
-  "Returns non-nil iff last Action Key use was a horizontal drag within a single window.
+  "Return non-nil iff last Action Key use was a horizontal drag within a single window.
 If free variable `assist-flag' is non-nil, uses Assist Key.
 Value returned is nil if not a horizontal drag, 'left if drag moved left or
 'right otherwise."
@@ -586,7 +593,7 @@ Value returned is nil if not a horizontal drag, 'left if drag moved left or
 	   (if (< last-depress-x last-release-x) 'right 'left)))))
 
 (defun hmouse-drag-vertically-within-emacs ()
-  "Returns non-nil iff last Action Key use was a vertical drag that started and ended within the same Emacs frame.
+  "Return non-nil iff last Action Key use was a vertical drag that started and ended within the same Emacs frame.
 If free variable `assist-flag' is non-nil, uses Assist Key.
 Value returned is nil if not a vertical line drag, 'up if drag moved up or
 'down otherwise."
@@ -615,7 +622,7 @@ Value returned is nil if not a vertical line drag, 'up if drag moved up or
 	   (if (< last-depress-y last-release-y) 'down 'up)))))
 
 (defun hmouse-drag-vertically ()
-  "Returns non-nil iff last Action Key use was a vertical drag within a single window.
+  "Return non-nil iff last Action Key use was a vertical drag within a single window.
 If free variable `assist-flag' is non-nil, uses Assist Key.
 Value returned is nil if not a vertical line drag, 'up if drag moved up or
 'down otherwise."
@@ -623,14 +630,9 @@ Value returned is nil if not a vertical line drag, 'up if drag moved up or
     (hmouse-drag-vertically-within-emacs)))
 
 (defun hmouse-drag-window-side ()
-  "Returns non-nil if Action Key was dragged from a window side divider and released in the same window.
+  "Return non-nil if Action Key was dragged from a window side divider and released in the same window.
 If free variable `assist-flag' is non-nil, uses Assist Key."
-  (cond ((featurep 'xemacs)
-	 ;; Depress events in scrollbars or in non-text area of buffer are
-	 ;; not visible or identifiable at the Lisp-level, so always return
-	 ;; nil.
-	 nil)
-	((hyperb:window-system)
+  (cond ((hyperb:window-system)
 	 (let* ((depress-args (if assist-flag assist-key-depress-args
 				action-key-depress-args))
 		(release-args (if assist-flag assist-key-release-args
@@ -647,14 +649,14 @@ If free variable `assist-flag' is non-nil, uses Assist Key."
 		    hmouse-side-sensitivity))))))
 
 (defun hmouse-read-only-toggle-key ()
-  "Return the first key binding that toggles read-only mode, or nil if none."
+  "Return the first key binding that toggle read-only mode, or nil if none."
   (key-description (or (where-is-internal #'read-only-mode nil t)
 		       (where-is-internal #'vc-toggle-read-only nil t)
 		       (where-is-internal #'toggle-read-only nil t))))
 
 (defun hmouse-vertical-action-drag ()
-  "Handles an Action Key vertical drag within a window: adds a window to the right of this one.
-Beeps and prints message if the window cannot be split further."
+  "Handle an Action Key vertical drag within a window: add a window to the right of this one.
+Beep and print message if the window cannot be split further."
   (interactive)
   (condition-case ()
       (split-window-horizontally nil)
@@ -662,16 +664,16 @@ Beeps and prints message if the window cannot be split further."
 	   (message "(hmouse-vertical-action-drag): Can't split the window further."))))
 
 (defun hmouse-vertical-assist-drag ()
-  "Handles an Assist Key vertical drag within a window: deletes the current window.
-Beeps and prints message if the window cannot be split further."
+  "Handle an Assist Key vertical drag within a window: deletes the current window.
+Beep and print message if the window cannot be split further."
   (condition-case ()
       (delete-window)
     (error (beep)
 	   (message "(hmouse-vertical-assist-drag): A single window cannot be deleted."))))
 
 (defun hmouse-horizontal-action-drag ()
-  "Handles an Action Key horizontal drag within a window: adds a window below this one.
-Beeps and prints message if the window cannot be split further."
+  "Handle an Action Key horizontal drag within a window: add a window below this one.
+Beep and print message if the window cannot be split further."
   (interactive)
   (condition-case ()
       (if (fboundp 'split-window-quietly)
@@ -681,60 +683,53 @@ Beeps and prints message if the window cannot be split further."
 	   (message "(hmouse-horizontal-action-drag): Can't split the window further."))))
 
 (defun hmouse-horizontal-assist-drag ()
-  "Handles an Assist Key horizontal drag within a window: deletes the current window.
-Beeps and prints message if the window cannot be split further."
+  "Handle an Assist Key horizontal drag within a window: delete the current window.
+Beep and print message if the window cannot be split further."
   (condition-case ()
       (delete-window)
     (error (beep)
 	   (message "(hmouse-horizontal-assist-drag): A single window cannot be deleted."))))
 
 (defun smart-coords-in-window-p (coords window)
-  "Tests if COORDS are in WINDOW.  Returns WINDOW if they are, nil otherwise."
+  "Test if COORDS are in WINDOW.  Return WINDOW if they are, nil otherwise."
   (cond ((null coords) nil)
-	((and (not (featurep 'xemacs)) (eventp coords))
+	((eventp coords)
 	 (let ((w-or-f (posn-window (event-start coords))))
-	   (if (framep w-or-f) (setq w-or-f (frame-selected-window w-or-f)))
-	   (eq w-or-f window)))
-	((if (featurep 'xemacs)
-	     (if (eventp coords)
-		 (eq (event-window coords) window)
-	       (eq (car coords) window))))
+	   (when (framep w-or-f)
+	     (setq w-or-f (frame-selected-window w-or-f)))
+	   (when (and (eq w-or-f window) (window-valid-p window))
+	     window)))
 	((fboundp 'window-edges)
 	 (let* ((edges (window-edges window))
-		  (w-xmin (nth 0 edges))
-		  (w-ymin (nth 1 edges))
-		  (w-xmax (nth 2 edges))
-		  (w-ymax (nth 3 edges))
-		  (x  (hmouse-x-coord coords))
-		  (y  (hmouse-y-coord coords)))
-	     (and (<= w-xmin x) (<= x w-xmax)
-		  (<= w-ymin y) (<= y w-ymax)
-		  window)))))
+		(w-xmin (nth 0 edges))
+		(w-ymin (nth 1 edges))
+		(w-xmax (nth 2 edges))
+		(w-ymax (nth 3 edges))
+		(x  (hmouse-x-coord coords))
+		(y  (hmouse-y-coord coords)))
+	   (and (<= w-xmin x) (<= x w-xmax)
+		(<= w-ymin y) (<= y w-ymax)
+		(window-valid-p window)
+		window)))))
 
 (defun smart-point-of-coords (coords)
-  "Returns point within window in which COORDS fall or nil if none.
+  "Return point within window in which COORDS fall or nil if none.
 Ignores minibuffer window."
   (cond ((markerp coords)
 	 (marker-position coords))
-	((and (not (featurep 'xemacs)) (eventp coords))
-	 (posn-point (event-start coords)))
-	((and (featurep 'xemacs) (eventp coords))
-	 (event-point coords))))
+	((eventp coords)
+	 (posn-point (event-start coords)))))
 
 (defun smart-window-of-coords (coords)
-  "Returns window in which COORDS fall or nil if none.
+  "Return window in which COORDS fall or nil if none.
 Ignores minibuffer window."
   (cond ((null coords) nil)
         ((markerp coords)
 	 (get-buffer-window (marker-buffer coords)))
-	((and (not (featurep 'xemacs)) (eventp coords))
+	((eventp coords)
 	 (let ((w-or-f (posn-window (event-start coords))))
 	   (if (framep w-or-f) (setq w-or-f (frame-selected-window w-or-f)))
 	   w-or-f))
-	((if (featurep 'xemacs)
-	     (if (eventp coords)
-		 (event-window coords)
-	       (car coords))))
 	(t (let ((window-list (hypb:window-list 'no-minibuf))
 		 (window)
 		 (w))
@@ -768,7 +763,7 @@ With optional boolean NEW-WINDOW non-nil, sensibly split the release window befo
   "Display an error when a region is active and in-window drags are not allowed."
   ;; Return point to where it was prior to depress so the region does not permanently change.
   (goto-char hkey-value)
-  (error "(hmouse-drag-region-active): Region is active; use a Smart Key press/click within a window, not a drag."))
+  (error "(hmouse-drag-region-active): Region is active; use a Smart Key press/click within a window, not a drag"))
 
 (defun hmouse-set-buffer-and-point (buffer point)
   (when buffer
@@ -826,6 +821,10 @@ With optional boolean NEW-WINDOW non-nil, sensibly split the release window befo
     (recenter)
     (pulse-momentary-highlight-one-line (point) 'next-error)))
 
+(defun hmouse-pulse-region (start end)
+  (when (and hmouse-pulse-flag (featurep 'pulse) (pulse-available-p))
+    (pulse-momentary-highlight-region start end 'next-error)))
+
 (defun hmouse-item-to-window (&optional new-window)
   "Display buffer or file menu item of Action Key depress at the location of Action Key release.
 
@@ -899,21 +898,26 @@ If the Action Key is:
      successive clicks walk up the directory tree
  (4) clicked anywhere in the middle of a window's modeline,
      the function given by `action-key-modeline-function' is called;
- (5) dragged vertically from modeline to within a window,
+ (5) dragged vertically from non-bottommost modeline to within a window,
      the modeline is moved to point of key release, thereby resizing
-     its window and potentially its vertical neighbors."
+     its window and potentially its vertical neighbors.
+ (6) dragged from a bottommost modeline in a frame with a non-nil
+     `drag-with-mode-line' parameter (use `frame-set-parameter'),
+     moves the frame as the drag occurs."
+  ;; Modeline window resizing is now handled in action-key-depress
+  ;; via a call to mouse-drag-mode-line, providing live visual
+  ;; feedback.
   (let ((w (smart-window-of-coords action-key-depress-args)))
     (if w (select-window w))
-    (cond ((hmouse-modeline-click)
-	   (cond ((hmouse-emacs-at-modeline-buffer-id-p)
-		  (funcall action-key-modeline-buffer-id-function))
-		 ((hmouse-release-left-edge) (bury-buffer))
-		 ((hmouse-release-right-edge)
-		  (if (eq major-mode 'Info-mode)
-		      (Info-exit)
-		    (info)))
-		 (t (funcall action-key-modeline-function))))
-	  (t (hmouse-modeline-resize-window)))))
+    (when (hmouse-modeline-click)
+      (cond ((hmouse-emacs-at-modeline-buffer-id-p)
+	     (funcall action-key-modeline-buffer-id-function))
+	    ((hmouse-release-left-edge) (bury-buffer))
+	    ((hmouse-release-right-edge)
+	     (if (eq major-mode 'Info-mode)
+		 (Info-exit)
+	       (info)))
+	    (t (funcall action-key-modeline-function))))))
 
 (defun assist-key-modeline ()
   "Handles Assist Key depresses on a window mode line.
@@ -928,13 +932,19 @@ If the Assist Key is:
      the next buffer in sequence is displayed in the window
  (4) clicked anywhere in the middle of a window's modeline,
      the function given by `assist-key-modeline-function' is called;
- (5) dragged vertically from modeline to within a window,
+ (5) dragged vertically from non-bottommost modeline to within a window,
      the modeline is moved to point of key release, thereby resizing
-     its window and potentially its vertical neighbors."
+     its window and potentially its vertical neighbors.
+ (6) dragged from a bottommost modeline in a frame with a non-nil
+     `drag-with-mode-line' parameter (use `frame-set-parameter'),
+     moves the frame as the drag occurs."
+  ;; Modeline window resizing is now handled in assist-key-depress
+  ;; via a call to mouse-drag-mode-line, providing live visual
+  ;; feedback.
   (let ((buffers)
 	(w (smart-window-of-coords assist-key-depress-args)))
-    (if w (select-window w))
-    (cond ((hmouse-modeline-click)
+    (when w (select-window w))
+    (when (hmouse-modeline-click)
 	   (cond ((hmouse-emacs-at-modeline-buffer-id-p)
 		  (next-buffer))
 		 ((hmouse-release-left-edge)
@@ -946,16 +956,34 @@ If the Assist Key is:
 		  (if (string-match "Hyperbole Smart Keys" (buffer-name))
 		      (hkey-help-hide)
 		    (hkey-summarize 'current-window)))
-		 (t (funcall assist-key-modeline-function))))
-	  (t (hmouse-modeline-resize-window)))))
+		 (t (funcall assist-key-modeline-function))))))
+
+(defun hmouse-drag-p ()
+  "Return non-nil if last Smart Key depress and release locations differ.
+Even if the mouse pointer stays within the same position within a
+window, its frame may have been moved by a bottommost modeline drag."
+  (not (and (eq (if assist-flag
+		    assist-key-depress-window
+		  action-key-depress-window)
+		(if assist-flag
+		    assist-key-release-window
+		  action-key-release-window))
+	    (equal (if assist-flag
+		       assist-key-depress-position
+		     action-key-depress-position)
+		   (if assist-flag
+		       assist-key-release-position
+		     action-key-release-position)))))
 
 (defun hmouse-modeline-click ()
-  "Returns non-nil if last Smart Key depress and release were at a single point (less than drag tolerance apart) in a modeline."
-  (and (hmouse-modeline-release) (hmouse-modeline-depress)
+  "Return non-nil if last Smart Key depress and release were at a single point (less than drag tolerance apart) in a modeline."
+  (and (not (hmouse-drag-p))
+       (hmouse-modeline-depress)
+       (hmouse-modeline-release)
        (not (or (hmouse-drag-horizontally) (hmouse-drag-vertically) (hmouse-drag-diagonally)))))
 
 (defun hmouse-emacs-modeline-event-p (event)
-  "GNU Emacs: Returns non-nil if EVENT happened on a window mode line."
+  "GNU Emacs: Return non-nil if EVENT happened on a window mode line."
   (or (and (eventp event) (eq (posn-area (event-start event)) 'mode-line))
       ;; If drag release was to an unselected frame mode-line, on
       ;; click-to-focus systems, the release event will not include
@@ -969,7 +997,7 @@ If the Assist Key is:
 	     last-press-y mode-ln (< (- mode-ln last-press-y) line-height)))))
 
 (defun hmouse-modeline-event-p (event)
-  "Returns non-nil if start of EVENT happened on a window mode line."
+  "Return non-nil if start of EVENT happened on a window mode line."
     (when (and (hyperb:window-system) event
 	       (not (posnp event))
 	       (not (markerp event)))
@@ -991,7 +1019,7 @@ If the Assist Key is:
 	  (and last-press-y mode-ln (= last-press-y mode-ln)))))))
 
 (defun hmouse-modeline-depress ()
-  "Returns non-nil if Action Key was depressed on a window mode line.
+  "Return non-nil if Action Key was depressed on a window mode line.
 If free variable `assist-flag' is non-nil, uses Assist Key."
   (let ((args (if assist-flag
 		  assist-key-depress-args
@@ -999,7 +1027,7 @@ If free variable `assist-flag' is non-nil, uses Assist Key."
     (hmouse-modeline-event-p args)))
 
 (defun hmouse-modeline-release ()
-  "Returns non-nil if Smart Key was released on a window mode line."
+  "Return non-nil if Smart Key was released on a window mode line."
   (let ((args (if assist-flag
 		  assist-key-release-args
 		action-key-release-args)))
@@ -1007,23 +1035,20 @@ If free variable `assist-flag' is non-nil, uses Assist Key."
 
 (defun hmouse-emacs-at-modeline-buffer-id-p ()
   "GNU Emacs: Return t if mouse position is within the buffer name field of the current window's mode-line, else nil."
-  (unless (featurep 'xemacs)
-    (let* ((coords (hmouse-window-coordinates)) ;; in characters
-	   (x-coord (caadr coords))
-	   (mode-line-string (and (integerp x-coord) (>= x-coord 0) (format-mode-line mode-line-format)))
-	   (keymap (and mode-line-string
-			(<= x-coord (1- (length mode-line-string)))
-			(plist-get (text-properties-at x-coord mode-line-string) 'local-map))))
-      (when keymap
-	(eq (lookup-key keymap [mode-line mouse-1]) 'mode-line-previous-buffer)))))
+  (let* ((coords (hmouse-window-coordinates)) ;; in characters
+	 (x-coord (caadr coords))
+	 (mode-line-string (and (integerp x-coord) (>= x-coord 0) (format-mode-line mode-line-format)))
+	 (keymap (and mode-line-string
+		      (<= x-coord (1- (length mode-line-string)))
+		      (plist-get (text-properties-at x-coord mode-line-string) 'local-map))))
+    (when keymap
+      (eq (lookup-key keymap [mode-line mouse-1]) 'mode-line-previous-buffer))))
 
 (defun hmouse-modeline-resize-window ()
   "Resizes window whose mode line was depressed on by the last Smart Key.
 Resize amount depends upon the vertical difference between press and release
 of the Smart Key."
   (cond ((not (hyperb:window-system)) nil)
-	((and (featurep 'xemacs) (not (fboundp 'window-edges)))
-	 (error "Drag from a mode-line with button1 to resize windows."))
 	(t (let* ((owind (selected-window))
 		  (window (smart-window-of-coords
 			   (if assist-flag assist-key-depress-args
@@ -1043,24 +1068,7 @@ of the Smart Key."
 			  (marker-buffer action-key-release-prev-point))
 			 (goto-char
 			  (marker-position action-key-release-prev-point)))
-		     (set-buffer obuf))))
-	     (cond
-	      ((>= (+ mode-ln 2) (frame-height))
-	       (error
-		"(hmouse-modeline-resize-window): Can't move bottom window in frame."))
-	      ((< (length (hypb:window-list 'no-minibuf)) 2)
-	       (error
-		"(hmouse-modeline-resize-window): Can't resize sole window in frame."))
-	      (t (unwind-protect
-		     (progn
-		       (select-window window)
-		       (shrink-window shrink-amount)
-		       ;; Keep redisplay from scrolling other window.
-		       (select-window (next-window nil 'no-mini))
-		       (condition-case ()
-			   (scroll-down shrink-amount)
-			 (error nil)))
-		   (select-window owind))))))))
+		     (set-buffer obuf))))))))
 
 (defun hmouse-clone-window-to-frame (&optional always-delete-flag)
   (let ((hycontrol-keep-window-flag t))
@@ -1101,7 +1109,7 @@ is non-nil, leave the original window and just clone it into the new frame."
 	       (delete-window depress-window)))))))
 
 (defun hmouse-release-left-edge ()
-  "Returns non-nil if last Smart Key release was at left window edge.
+  "Return non-nil if last Smart Key release was at left window edge.
 `hmouse-edge-sensitivity' value determines how near to actual edge the
 release must be.
 
@@ -1112,31 +1120,21 @@ mode-line regardless of the setting of `hmouse-edge-sensitivity'."
   (let ((args (if assist-flag assist-key-release-args
 		 action-key-release-args))
 	window-left last-release-x)
-    (if (fboundp 'window-lowest-p) ;; XEmacs >= 19.12 
-	(setq last-release-x (and args (eq (event-window args)
-					   (selected-window))
-				  (hmouse-x-coord args))
-	      window-left 0)
-      (setq window-left (car (window-edges))
-	    last-release-x (and args (hmouse-x-coord args))))
+    (setq window-left (car (window-edges))
+	  last-release-x (and args (hmouse-x-coord args)))
     (and last-release-x (< (- last-release-x window-left)
 			   hmouse-edge-sensitivity)
 	 (>= (- last-release-x window-left) 0))))
 
 (defun hmouse-release-right-edge ()
-  "Returns non-nil if the last Smart Key release was at right window edge.
+  "Return non-nil if the last Smart Key release was at right window edge.
 `hmouse-edge-sensitivity' value determines how near to actual edge the
 release must be."
   (let ((args (if assist-flag assist-key-release-args
 		 action-key-release-args))
 	window-right last-release-x)
-    (if (fboundp 'window-lowest-p) ;; XEmacs >= 19.12 
-	(setq last-release-x (and args (eq (event-window args)
-					   (selected-window))
-				  (hmouse-x-coord args))
-	      window-right (window-width))
-      (setq window-right (nth 2 (window-edges))
-	    last-release-x (and args (hmouse-x-coord args))))
+    (setq window-right (nth 2 (window-edges))
+	  last-release-x (and args (hmouse-x-coord args)))
     (and last-release-x (>= (+ last-release-x hmouse-edge-sensitivity)
 			    window-right)
 	 (>= (- window-right last-release-x) 0))))
@@ -1145,12 +1143,7 @@ release must be."
   "Resizes window whose side was depressed on by the last Smart Key.
 Resize amount depends upon the horizontal difference between press and release
 of the Smart Key."
-  (cond ((featurep 'xemacs)
-	 ;; Depress events in scrollbars or in non-text area of buffer are
-	 ;; not visible or identifiable at the Lisp-level, so always return
-	 ;; nil.
-	 nil)
-	((hyperb:window-system)
+  (cond ((hyperb:window-system)
 	 (let* ((owind (selected-window))
 		(window (smart-window-of-coords
 			 (if assist-flag assist-key-depress-args
@@ -1173,10 +1166,10 @@ of the Smart Key."
 	   (cond
 	    ((>= (+ right-side-ln 2) (frame-width))
 	     (error
-	      "(hmouse-resize-window-side): Can't change width of full frame width window."))
+	      "(hmouse-resize-window-side): Can't change width of full frame width window"))
 	    ((< (length (hypb:window-list 'no-minibuf)) 2)
 	     (error
-	      "(hmouse-resize-window-side): Can't resize sole window in frame."))
+	      "(hmouse-resize-window-side): Can't resize sole window in frame"))
 	    (t (unwind-protect
 		   (progn
 		     (select-window window)
@@ -1193,7 +1186,7 @@ of the Smart Key."
 	 (w2-buf (and w2 (window-buffer w2)))
 	 )
     (cond ((not (and w1 w2))
-	   (error "(hmouse-swap-buffers): Last depress or release was not within a window."))
+	   (error "(hmouse-swap-buffers): Last depress or release was not within a window"))
 	  ((eq w1 w2)
 	   ;; Do nothing silently.
 	   )
@@ -1213,7 +1206,7 @@ of the Smart Key."
 	 (w2-height (and w2 (window-height w2)))
 	 )
     (or (and w1 w2)
-	(error "(hmouse-swap-windows): Last depress or release was not within a window."))
+	(error "(hmouse-swap-windows): Last depress or release was not within a window"))
     (unwind-protect
 	(progn
 	  (select-window w1)
@@ -1233,7 +1226,7 @@ of the Smart Key."
       )))
 
 (defun hmouse-x-coord (args)
-  "Returns x coordinate in characters from window system dependent ARGS or nil."
+  "Return x coordinate in characters from window system dependent ARGS or nil."
   (let ((x (if (markerp args)
 	       (save-excursion
 		 (hypb:goto-marker args)
@@ -1248,16 +1241,12 @@ of the Smart Key."
 						       (+ (car (posn-col-row args))
 							  (nth 0 (window-edges w-or-f)))))
 						    (t (car args)))))
-				 ("xemacs" .  (if (eventp args)
-						  (event-x args)
-						(car args)))
-				 ("xterm"  .  (car args))
 				 ("next"   .  (nth 1 args))
 				 )))))))
     (if (integerp x) x)))
 
 (defun hmouse-y-coord (args)
-  "Returns y coordinate in frame lines from window system dependent ARGS or nil."
+  "Return y coordinate in frame lines from window system dependent ARGS or nil."
   (let ((y (eval (cdr (assoc (hyperb:window-system)
 			     '(("emacs" . (progn (if (eventp args) (setq args (event-start args)))
 						 (cond ((posnp args)
@@ -1267,10 +1256,6 @@ of the Smart Key."
 							  (+ (cdr (posn-col-row args))
 							     (nth 1 (window-edges w-or-f)))))
 						       (t (cdr args)))))
-			       ("xemacs" .  (if (eventp args)
-						(event-y args)
-					      (cdr args)))
-			       ("xterm"  .  (nth 1 args))
 			       ("next"   .  (nth 2 args))
 			       ))))))
     (if (integerp y) y)))
